/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package main;

import static opengl.GL.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.Display;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;
import util.Util;

/**
 *
 * @author NMARNIOK
 */
public class TranslateAndRotateNgon {
    private static int programID;       // shader program id
    private static int ngonVAID;        // vertexarray id of the n-gon
    private static final Vector3f translation = new Vector3f();
    private static float rotation = 0;
    private static final Matrix4f tempMat = new Matrix4f();
    
    public static void main(String[] argv) {
        try {
            init();
            programID = Util.createShaderProgram("shader/MainTR_VS.glsl", "shader/MainBrightness_FS.glsl");
            ngonVAID = Util.createNGon(12);
            glPointSize(16.0f);
            render();
            Display.destroy();
        } catch (LWJGLException ex) {
            Logger.getLogger(TranslateAndRotateNgon.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    public static void render() throws LWJGLException {
        glClearColor(1.0f, 0.0f, 0.0f, 1.0f);
        while(!Display.isCloseRequested()) {
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
            glUseProgram(programID);
            glBindVertexArray(ngonVAID);
            glDrawElements(GL_POINTS, 12, GL_UNSIGNED_INT, 0);
            Display.update();
            Display.sync(60);
            
            handleInput();
            updateUniforms();
        }
    }
    
    public static void handleInput() {
        float speed = 1e-2f * (Keyboard.isKeyDown(Keyboard.KEY_LSHIFT) ? 2.0f : 1.0f);
        if(Keyboard.isKeyDown(Keyboard.KEY_W)) translation.y = Math.max(-1.0f, Math.min(translation.y+speed, 1.0f));
        if(Keyboard.isKeyDown(Keyboard.KEY_S)) translation.y = Math.max(-1.0f, Math.min(translation.y-speed, 1.0f));
        if(Keyboard.isKeyDown(Keyboard.KEY_A)) translation.x = Math.max(-1.0f, Math.min(translation.x-speed, 1.0f));
        if(Keyboard.isKeyDown(Keyboard.KEY_D)) translation.x = Math.max(-1.0f, Math.min(translation.x+speed, 1.0f));
        
        if(Keyboard.isKeyDown(Keyboard.KEY_Q)) rotation += speed;
        if(Keyboard.isKeyDown(Keyboard.KEY_E)) rotation -= speed;
    }
    
    public static void updateUniforms() {
        // translate
        tempMat.setIdentity();
        tempMat.m30 = translation.x;
        tempMat.m31 = translation.y;
        tempMat.m32 = translation.z;
        pushMatrixToUniform("translate");
        
        // rotate
        tempMat.setIdentity();
        tempMat.m00 = tempMat.m11 = (float)Math.cos(rotation);
        tempMat.m10 = -(tempMat.m01 = (float)Math.sin(rotation));
        pushMatrixToUniform("rotate");
        
        // light direction
        int lightLoc = glGetUniformLocation(programID, "inverseLightDir");
        glUniform3f(lightLoc, 0.0f, 1.0f, 0.0f);
    }
    
    private static void pushMatrixToUniform(String uniformName) {
        tempMat.store(Util.MAT_BUFFER);
        Util.MAT_BUFFER.position(0);
        int invScaleLoc = glGetUniformLocation(programID, uniformName);
        glUniformMatrix4(invScaleLoc, false, Util.MAT_BUFFER);        
    }
}
